'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

exports.default = graphqlModule;

var _underscore = require('underscore');

var _underscore2 = _interopRequireDefault(_underscore);

var _graphqlTag = require('graphql-tag');

var _graphqlTag2 = _interopRequireDefault(_graphqlTag);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function graphqlModule(app) {
  // 新增 app.graphqlFragments
  app.graphqlFragments = {};

  _underscore2.default.extend(app.constructor.prototype, {
    // 註冊 fragment
    registerGraphqlFragment: function registerGraphqlFragment(fragmentTextSource) {
      // 先移除 comment
      var fragmentText = fragmentTextSource.replace(/#.*\n/g, '\n');

      // 讀取 fragment name
      var fragmentName = fragmentText.match(/fragment (.*) on/)[1];

      // 讀取 sub fragments 字串
      var matchedSubFragments = fragmentText.match(/\.{3}([_A-Za-z][_0-9A-Za-z]*)/g) || [];
      var subFragments = _underscore2.default.unique(matchedSubFragments.map(function (f) {
        return f.replace('...', '');
      }));

      // 將 fragment 資料儲存到 graphqlFragments
      this.graphqlFragments[fragmentName] = {
        fragmentText: fragmentText,
        subFragments: subFragments
      };
    },


    // extend fragment
    extendGraphqlFragment: function extendGraphqlFragment(fragmentName, newProperties) {
      // 由 graphqlFragments 讀取 fragment 資料
      var fragment = this.graphqlFragments[fragmentName];
      var fragmentEndPosition = fragment.fragmentText.lastIndexOf('}');
      // 將新 properties 插到最後
      var newFragmentText = [fragment.fragmentText.slice(0, fragmentEndPosition), newProperties, fragment.fragmentText.slice(fragmentEndPosition)].join('');
      // 重新註冊 fragment
      this.registerGraphqlFragment(newFragmentText);
    },


    // 讀取 fragment object
    getGraphqlFragment: function getGraphqlFragment(fragmentName) {
      var _this = this;

      // 先確保 graphqlFragments 有 fragment 名稱
      var fragment = this.graphqlFragments[fragmentName];

      if (!fragment) {
        throw new Error('Fragment "' + fragmentName + '" not registered.');
      }

      // 如果之前已經產生過, 即可以返回了
      if (fragment.fragmentObject) {
        return fragment.fragmentObject;
      }

      // 由 graphql-tag 產生 fragment object 並返回
      var literals = [fragment.fragmentText].concat((0, _toConsumableArray3.default)(fragment.subFragments.map(function (x) {
        return '\n';
      })));
      var gqlArguments = [literals].concat((0, _toConsumableArray3.default)(fragment.subFragments.map(function (f) {
        return _this.getGraphqlFragment(f, fragmentName);
      })));
      fragment.fragmentObject = _graphqlTag2.default.apply(null, gqlArguments);
      return fragment.fragmentObject;
    }
  });
}