'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

exports.createApolloClient = createApolloClient;

var _apolloClient = require('apollo-client');

var _apolloClient2 = _interopRequireDefault(_apolloClient);

var _apolloLink = require('apollo-link');

var _apolloLinkHttp = require('apollo-link-http');

var _apolloCacheInmemory = require('apollo-cache-inmemory');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function createApolloClient(_ref) {
  var initialState = _ref.initialState,
      ssrForceFetchDelay = _ref.ssrForceFetchDelay,
      graphqlUrl = _ref.graphqlUrl,
      loginToken = _ref.loginToken,
      headerLoginTokenKey = _ref.headerLoginTokenKey,
      localStorageLoginTokenKey = _ref.localStorageLoginTokenKey;

  var cache = new _apolloCacheInmemory.InMemoryCache({
    addTypename: true
  });

  var httpLink = (0, _apolloLinkHttp.createHttpLink)({ uri: graphqlUrl });
  var middlewareLink = new _apolloLink.ApolloLink(function (operation, forward) {
    // 若為 client side, 讀取 localStorage 中的 login token
    var localStorageLoginToken = typeof window !== 'undefined' && window.localStorage && window.localStorage.getItem(localStorageLoginTokenKey);

    // client side 會選 localStorageLoginToken, server side 會選 cookie 中的 loginToken
    loginToken = localStorageLoginToken || loginToken;

    // 傳送 loginToken 到認證用 header
    operation.setContext({
      headers: (0, _defineProperty3.default)({}, headerLoginTokenKey, loginToken || null)
    });

    // forward 給 httpLink 處理
    return forward(operation);
  });
  var link = middlewareLink.concat(httpLink);

  return new _apolloClient2.default({
    link: link,
    cache: cache.restore(initialState || {}),
    ssrMode: typeof window === 'undefined',
    queryDeduplication: true,
    ssrForceFetchDelay: ssrForceFetchDelay
  });
}