'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

exports.createApollolServer = createApollolServer;

var _extend = require('extend');

var _extend2 = _interopRequireDefault(_extend);

var _express = require('express');

var _express2 = _interopRequireDefault(_express);

var _bodyParser = require('body-parser');

var _bodyParser2 = _interopRequireDefault(_bodyParser);

var _graphql = require('graphql');

var _graphqlTools = require('graphql-tools');

var _apolloServerExpress = require('apollo-server-express');

var _cors = require('cors');

var _cors2 = _interopRequireDefault(_cors);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function createApollolServer(server) {
  var _this = this;

  var config = {
    // 若 settings 沒有 graphqlPath, 則 path 為 /graphql
    path: server.getSetting('graphqlPath', '/graphql'),
    // 若 settings 沒有 graphiql, enable graphiql 由 server.dev 決定
    graphiql: server.getSetting('graphiql', server.dev),
    // 若 settings 沒有 graphiqlPath, graphiql path 為 /graphiql
    graphiqlPath: server.getSetting('graphiqlPath', '/graphiql'),
    // graphiql 自動傳送 localstorage 的 loginToken
    graphiqlOptions: {
      passHeader: '"' + server.getSetting('headerLoginTokenKey') + '": localStorage["' + server.getSetting('localStorageLoginTokenKey') + '"] || ""'
    }

    // 產生 executable schema
  };var schema = generateExecutableSchema({
    schemaObj: server.graphqlSchemaObj,
    resolvers: server.graphqlResolvers
  });

  // 暫存 server._typeDefs
  server._typeDefs = schema._typeDefs;

  // 建立 graphqlExpress 所需的 options
  var options = {
    formatError: function formatError(e) {
      return {
        message: e.message,
        locations: e.locations,
        path: e.path
      };
    },
    debug: server.dev,
    schema: schema,
    context: server.graphqlContext

    // graphqlServer
  };var graphqlServer = (0, _express2.default)();

  // 將 rooturl 加到 whitelist
  var whitelist = [server.absoluteUrl().replace(/\/$/, '')];
  // 繼續由 settings 去 custom whitelist
  var graphqlServerCORSWhitelist = server.getSetting('graphqlServerCORSWhitelist');
  if (graphqlServerCORSWhitelist) {
    whitelist = [].concat((0, _toConsumableArray3.default)(whitelist), (0, _toConsumableArray3.default)(graphqlServerCORSWhitelist));
  }

  // graphql server CORS support
  // match 到 whitelist 其中一個就可以了
  var corsOptions = {
    origin: function origin(_origin, callback) {
      var originIsWhitelisted = whitelist.indexOf(_origin) !== -1;
      if (!originIsWhitelisted) {
        for (var i = 0; i < whitelist.length; i++) {
          var pattern = whitelist[i];
          try {
            var isMatch = new RegExp(pattern).test(_origin);
            if (isMatch) {
              originIsWhitelisted = true;
              break;
            }
          } catch (__) {}
        }
      }
      callback(null, originIsWhitelisted);
    },

    credentials: true,
    optionsSuccessStatus: 200
  };
  graphqlServer.use((0, _cors2.default)(corsOptions));

  // 繼續設定 graphqlServer
  graphqlServer.use(config.path, _bodyParser2.default.json(), (0, _apolloServerExpress.graphqlExpress)(function () {
    var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(req) {
      var loginToken, newContext;
      return _regenerator2.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              _context.prev = 0;

              // 讀取傳過來 req.headers 的 loginToken
              loginToken = req.headers && req.headers[server.getSetting('headerLoginTokenKey')];
              // 針對現有的 options.context 進行 custom, loginToken 作為 context
              // 註: runCallbacksAsync

              _context.next = 4;
              return server.runCallbacksAsync('graphql.context', options.context, { loginToken: loginToken });

            case 4:
              newContext = _context.sent;
              return _context.abrupt('return', (0, _extends3.default)({}, options, {
                context: newContext
              }));

            case 8:
              _context.prev = 8;
              _context.t0 = _context['catch'](0);

              // 過程中有任何錯誤僅印出來算了
              console.error('[Apollo Integration] Something bad happened' + 'when handling a request on the GraphQL server. ' + 'Your GraphQL server is not working as expected:', _context.t0);
              return _context.abrupt('return', options);

            case 12:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, _this, [[0, 8]]);
    }));

    return function (_x) {
      return _ref.apply(this, arguments);
    };
  }()));

  // 若要開啟 graphiql
  if (config.graphiql) {
    graphqlServer.use(config.graphiqlPath, (0, _apolloServerExpress.graphiqlExpress)((0, _extends3.default)({}, config.graphiqlOptions, {
      // endpointURL 可由 graphqlEndpointURL 設置
      endpointURL: server.getSetting('graphqlEndpointURL', config.path)
    })));
  }

  return graphqlServer;
}

// 產生 executable schema
function generateExecutableSchema(_ref2) {
  var schemaObj = _ref2.schemaObj,
      resolvers = _ref2.resolvers;

  // 若 query fields 中什麼也沒有, 新增 hello world
  if (!schemaObj.queryFields.length) {
    schemaObj.queryFields.push('\n      # say hello\n      hello: String\n    ');

    (0, _extend2.default)(true, resolvers, { Query: {
        hello: function hello() {
          return 'Hello World!';
        }
      } });
  }

  // 暫存
  var typeDefs = [''];

  // 將 string array 內的 strings 加在一起
  var schemasText = schemaObj.schemas.join('\n');
  var queriesText = schemaObj.queryFields.join('\n');
  var mutationsText = schemaObj.mutationFields.join('\n');

  // 建立總 typeDefs
  typeDefs[0] = '\n    scalar JSON\n    scalar Date\n\n    ' + schemasText + '\n\n    ' + (schemaObj.queryFields.length > 0 ? '\n    type Query {\n      ' + queriesText + '\n    }\n    ' : '') + '\n\n    ' + (schemaObj.mutationFields.length > 0 ? '\n    type Mutation {\n      ' + mutationsText + '\n    }\n    ' : '') + '\n  ';

  // 建立 schema
  var schema = (0, _graphqlTools.makeExecutableSchema)({
    typeDefs: typeDefs,
    resolvers: resolvers
  });

  // schema._typeDefs 為好看版的 typeDefs
  schema._typeDefs = [(0, _graphql.print)((0, _graphql.parse)(typeDefs[0]))];

  return schema;
}