'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _http = require('http');

var _http2 = _interopRequireDefault(_http);

var _express = require('express');

var _express2 = _interopRequireDefault(_express);

var _cookieParser = require('cookie-parser');

var _cookieParser2 = _interopRequireDefault(_cookieParser);

var _isomorphicFetch = require('isomorphic-fetch');

var _isomorphicFetch2 = _interopRequireDefault(_isomorphicFetch);

require('vclass-lib');

var _index = require('./modules/index.js');

var _apolloServer = require('./apollo-server.js');

var _static = require('./static.js');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// 載入 vclass-lib 在 server-side 要用到的 node modules, 如 url 和 crypto
global.fetch = _isomorphicFetch2.default;

// 先 import server 所有 modules, 如 callbacks, settings
require('./modules/server.js');

var Server = function () {
  function Server() {
    var _this = this;

    var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    (0, _classCallCheck3.default)(this, Server);

    options.dir = options.dir || '.';
    options.dev = options.dev || false;

    this.dir = options.dir;
    this.dev = options.dev;

    this.handleRequest = (0, _express2.default)();

    // 在 server constructor 階段中, 執行所有 server modules
    // 每個 module 傳入兩個變數
    // - this: server instance
    // - ctx: 為 options
    // 所有 modules function 為 sync function
    var ctx = (0, _extends3.default)({}, options);
    _index.modules.forEach(function (module) {
      if (typeof module === 'function') {
        module(_this, ctx);
      }
    });
  }

  (0, _createClass3.default)(Server, [{
    key: 'getRequestHandler',
    value: function getRequestHandler() {
      return this.handleRequest;
    }

    // prepare return promise, 需要上層作錯誤處理

  }, {
    key: 'prepare',
    value: function () {
      var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2() {
        var mongoPort, fs, _require, MongodHelper, mongodHelper, startupHooks, hook, server, readyHooks, _hook;

        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                if (process.env.MONGO_URL) {
                  _context2.next = 23;
                  break;
                }

                // 若 process.env.MONGO_PORT 沒有設置，當 mongoPort 為 3001
                mongoPort = process.env.MONGO_PORT || 3001;
                // 設定 process.env.MONGO_URL 為往後的 Mongo.Collection 作準備

                process.env.MONGO_URL = 'mongodb://127.0.0.1:' + mongoPort + '/vclass';

                // 確保有 db 資料夾
                fs = require('fs-extra');
                _context2.prev = 4;

                fs.ensureDirSync('./.vclass/db');
                _context2.next = 11;
                break;

              case 8:
                _context2.prev = 8;
                _context2.t0 = _context2['catch'](4);
                throw _context2.t0;

              case 11:

                // 建立 mongod 服務
                _require = require('vclass-lib/mongodb-prebuilt'), MongodHelper = _require.MongodHelper;
                mongodHelper = new MongodHelper(['--port', '' + mongoPort, '--dbpath', './.vclass/db']);
                _context2.prev = 13;
                _context2.next = 16;
                return mongodHelper.run();

              case 16:
                console.log('mongod is running');
                _context2.next = 23;
                break;

              case 19:
                _context2.prev = 19;
                _context2.t1 = _context2['catch'](13);

                console.log('error starting', _context2.t1);
                throw _context2.t1;

              case 23:

                // 保留 _startup 作 debug 用
                this.callbacks._startup = [].concat((0, _toConsumableArray3.default)(this.callbacks.startup || []));
                // server side 的 startupHooks 裡可以放 async startup callbacks
                // 將 server instance(this) 傳給 hook 作為變數
                startupHooks = this.callbacks.startup || [];

              case 25:
                if (!startupHooks.length) {
                  _context2.next = 31;
                  break;
                }

                hook = startupHooks.shift();
                _context2.next = 29;
                return hook(this);

              case 29:
                _context2.next = 25;
                break;

              case 31:

                // 暫存
                server = this;

                // startup callbacks 後
                // 可以開始各種服務, 如 static 和 apolloserver, cookieParser
                // 將各種服務加到 handleRequest
                // static file server

                this.staticServer = (0, _static.createStaticServer)();
                this.handleRequest.use(this.staticServer);
                // apollo server
                this.graphqlServer = (0, _apolloServer.createApollolServer)(this);
                this.handleRequest.use(this.graphqlServer);
                // cookie parser
                this.handleRequest.use((0, _cookieParser2.default)());
                // 將 vclass object 加到 req
                this.handleRequest.use(function () {
                  var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(req, res, next) {
                    var loginToken;
                    return _regenerator2.default.wrap(function _callee$(_context) {
                      while (1) {
                        switch (_context.prev = _context.next) {
                          case 0:
                            _context.prev = 0;

                            // 將 server 加到 req.vclass
                            req.vclass = { server: server
                              // res.vclass 和 req.vclass 一樣
                            };res.vclass = req.vclass;

                            // 若果 req.url 不是 app url, 即忽略

                            if (!(req.url.indexOf('/graphql') === 0 || req.url.indexOf('/_next') === 0 || req.url.indexOf('/_oauth') === 0)) {
                              _context.next = 6;
                              break;
                            }

                            next();
                            return _context.abrupt('return');

                          case 6:

                            // req.url 為 app url
                            // 由 req 讀取 cookies loginToken
                            loginToken = req.cookies && req.cookies[server.getSetting('cookiesLoginTokenKey')];

                            // 讓 callback 幫助 custom req.vclass, 將 loginToken 和各種有用資訊作為 context
                            // 當中這些 callback 極有可能做用戶認證等資料庫操作，所以 run callbacks 時要註意是 async

                            _context.next = 9;
                            return server.runCallbacksAsync('req.vclass', req.vclass, { loginToken: loginToken, req: req, res: res });

                          case 9:
                            req.vclass = _context.sent;

                            res.vclass = req.vclass;

                            next();
                            _context.next = 18;
                            break;

                          case 14:
                            _context.prev = 14;
                            _context.t0 = _context['catch'](0);

                            console.log('handleRequest error', _context.t0);
                            next();

                          case 18:
                          case 'end':
                            return _context.stop();
                        }
                      }
                    }, _callee, this, [[0, 14]]);
                  }));

                  function injectToRequest(_x2, _x3, _x4) {
                    return _ref2.apply(this, arguments);
                  }

                  return injectToRequest;
                }());

                // 保留 _ready 作 debug 用
                this.callbacks._ready = [].concat((0, _toConsumableArray3.default)(this.callbacks.ready || []));
                // server side 的 readyHooks 裡可以放 async startup callbacks
                // 將 server instance(this) 傳給 hook 作為變數
                readyHooks = this.callbacks.ready || [];

              case 40:
                if (!readyHooks.length) {
                  _context2.next = 46;
                  break;
                }

                _hook = readyHooks.shift();
                _context2.next = 44;
                return _hook(this);

              case 44:
                _context2.next = 40;
                break;

              case 46:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this, [[4, 8], [13, 19]]);
      }));

      function prepare() {
        return _ref.apply(this, arguments);
      }

      return prepare;
    }()

    // prepare return promise, 需要上層作錯誤處理

  }, {
    key: 'start',
    value: function () {
      var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(port, hostname) {
        var _this2 = this;

        var next, nextApp, nextHandle, vclassHandle, app;
        return _regenerator2.default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                // 建立 next app
                next = require('next');
                nextApp = next({ dir: this.dir, dev: this.dev });

                // 宣告各個 handler

                nextHandle = nextApp.getRequestHandler();
                vclassHandle = this.getRequestHandler();

                // 準備好

                _context3.next = 6;
                return this.prepare();

              case 6:
                _context3.next = 8;
                return nextApp.prepare();

              case 8:

                // express
                app = (0, _express2.default)();
                // 先 use 本 server 的 handler

                app.use(vclassHandle);
                // 其他 request 交給 next 控制
                app.get('*', function (req, res) {
                  return nextHandle(req, res);
                });
                // 建立 http server
                this.http = _http2.default.createServer(app);

                // 用於檢查 port 是否 already in use
                _context3.next = 14;
                return new _promise2.default(function (resolve, reject) {
                  // This code catches EADDRINUSE error if the port is already in use
                  _this2.http.on('error', reject);
                  _this2.http.on('listening', function () {
                    return resolve();
                  });
                  _this2.http.listen(port, hostname);
                });

              case 14:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function start(_x5, _x6) {
        return _ref3.apply(this, arguments);
      }

      return start;
    }()

    // 新增 startup callback

  }, {
    key: 'startup',
    value: function startup(callback, order) {
      this.addCallback('startup', callback, order);
    }

    // 新增 ready callback

  }, {
    key: 'ready',
    value: function ready(callback, order) {
      this.addCallback('ready', callback, order);
    }
  }]);
  return Server;
}();

exports.default = Server;