'use strict';

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _underscore = require('underscore');

var _underscore2 = _interopRequireDefault(_underscore);

var _schema = require('vclass-lib/schema');

var _mongo = require('vclass-lib/mongo');

var _index = require('./index.js');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// 新增 schema
_mongo.Mongo.Collection.prototype.attachSchema = function (schemaOrFields) {
  if (schemaOrFields instanceof _schema.Schema) {
    this.schema = schemaOrFields;
  } else {
    this.schema.extend(schemaOrFields);
  }
};

// 讀取 schema
_mongo.Mongo.Collection.prototype.getSchema = function () {
  return this.schema;
};

// 讀取 schema object
_mongo.Mongo.Collection.prototype.getSchemaObject = function () {
  if (!this.schema) return null;
  return this.schema.schema();
};

// 新增 field
_mongo.Mongo.Collection.prototype.addField = function (fieldOrFieldArray) {
  var schemaObj = this.getSchemaObject();
  var fieldSchema = {};
  var fieldArray = Array.isArray(fieldOrFieldArray) ? fieldOrFieldArray : [fieldOrFieldArray];
  fieldArray.forEach(function (field) {
    var newField = (0, _extends3.default)({}, schemaObj[field.fieldName], field.fieldSchema);
    fieldSchema[field.fieldName] = newField;
  });
  this.attachSchema(fieldSchema);
};

// 移除 field
_mongo.Mongo.Collection.prototype.removeField = function (fieldName) {
  var schemaObj = _underscore2.default.omit(this.getSchemaObject(), fieldName);
  this.attachSchema(new _schema.Schema(schemaObj));
};

function collectionsModule(server) {
  // 新增 server.collections
  server.collections = {};

  _underscore2.default.extend(server.constructor.prototype, {
    // 建立並註冊 collection
    createCollection: function createCollection(name) {
      var collection = new _mongo.Mongo.Collection(name);
      return this.registerCollection(name, collection);
    },

    // 註冊 collection, 登記到 server.collections
    registerCollection: function registerCollection(name, collection) {
      this.collections[name] = collection;
      return collection;
    }
  });
}

(0, _index.addModule)(collectionsModule);