'use strict';

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _underscore = require('underscore');

var _underscore2 = _interopRequireDefault(_underscore);

var _ejson = require('vclass-lib/ejson');

var _index = require('./index.js');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function methodsModule(server) {
  // 新增 server.methodHandlers
  server.methodHandlers = {};

  // 新增 mutation field - methodInvoker
  server.addGraphqlMutationField('\n    # methodInvoker\n    methodInvoker (message: JSON): JSON\n  ');

  // resolvers
  server.addGraphqlResolvers({
    Mutation: {
      methodInvoker: function () {
        var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(root, _ref, context) {
          var message = _ref.message;
          var result;
          return _regenerator2.default.wrap(function _callee$(_context) {
            while (1) {
              switch (_context.prev = _context.next) {
                case 0:
                  _context.next = 2;
                  return processMessage(message);

                case 2:
                  result = _context.sent;

                  try {
                    result = JSON.parse((0, _stringify2.default)(result));
                  } catch (__) {}
                  return _context.abrupt('return', result);

                case 5:
                case 'end':
                  return _context.stop();
              }
            }
          }, _callee, this);
        }));

        function methodInvoker(_x, _x2, _x3) {
          return _ref2.apply(this, arguments);
        }

        return methodInvoker;
      }()
    }
  });

  var processMessage = function () {
    var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(msg) {
      var result;
      return _regenerator2.default.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              if (msg) {
                _context2.next = 2;
                break;
              }

              return _context2.abrupt('return');

            case 2:
              if (!_underscore2.default.has(protocolHandlers, msg.msg)) {
                _context2.next = 8;
                break;
              }

              _context2.next = 5;
              return protocolHandlers[msg.msg](msg);

            case 5:
              result = _context2.sent;
              _context2.next = 9;
              break;

            case 8:
              result = getErrorMessage('Bad request', msg);

            case 9:
              return _context2.abrupt('return', result);

            case 10:
            case 'end':
              return _context2.stop();
          }
        }
      }, _callee2, this);
    }));

    return function processMessage(_x4) {
      return _ref3.apply(this, arguments);
    };
  }();

  var protocolHandlers = {
    method: function () {
      var _ref4 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(msg) {
        var handler, payload, result;
        return _regenerator2.default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                if (!(typeof msg.method !== 'string' || 'params' in msg && !(msg.params instanceof Array))) {
                  _context3.next = 2;
                  break;
                }

                return _context3.abrupt('return', getErrorMessage('Malformed method invocation', msg));

              case 2:

                // 讀取 handler
                handler = server.methodHandlers[msg.method];

                if (handler) {
                  _context3.next = 5;
                  break;
                }

                return _context3.abrupt('return', {
                  msg: 'result',
                  error: new server.Error(404, 'Method \'' + msg.method + '\' not found')
                });

              case 5:

                // 準備 payload result
                payload = {
                  msg: 'result'
                };
                _context3.prev = 6;
                _context3.next = 9;
                return handler.apply(server, msg.params);

              case 9:
                result = _context3.sent;

                // 儲存到 payload
                if (result !== undefined) {
                  payload.result = result;
                }
                _context3.next = 16;
                break;

              case 13:
                _context3.prev = 13;
                _context3.t0 = _context3['catch'](6);

                // 若有錯誤, payload 新增 error
                payload.error = wrapInternalException(_context3.t0, 'while invoking method \'' + msg.method + '\'');

              case 16:
                return _context3.abrupt('return', payload);

              case 17:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this, [[6, 13]]);
      }));

      function method(_x5) {
        return _ref4.apply(this, arguments);
      }

      return method;
    }()
  };

  // 當 handler 出錯時會用 wrapInternalException 包住 result
  var wrapInternalException = function wrapInternalException(exception, context) {
    // 返回 server.Error
    if (!exception || exception instanceof server.Error) {
      return exception;
    }

    // 輸出錯誤訊息
    if (!exception.expected) {
      console.log('Exception ' + context, exception.stack);
      if (exception.sanitizedError) {
        console.log('Sanitized and reported to the client as:', exception.sanitizedError.message);
        console.log();
      }
    }

    // 返回 exception.sanitizedError
    if (exception.sanitizedError) {
      if (exception.sanitizedError instanceof server.Error) {
        return exception.sanitizedError;
      }
      console.log('Exception ' + context + ' provides a sanitizedError that ' + 'is not a server.Error; ignoring');
    }

    // 返回 server.Error 500 error
    return new server.Error(500, 'Internal server error');
  };

  // 用於輸出錯誤 result, 當 msg 格式不對時會用上
  var getErrorMessage = function getErrorMessage(reason, offendingMessage) {
    var msg = { msg: 'error', reason: reason };
    if (offendingMessage) {
      msg.offendingMessage = offendingMessage;
    }
    return msg;
  };

  _underscore2.default.extend(server.constructor.prototype, {
    addMethods: function addMethods(methods) {
      var _this = this;

      // 新增 method handler 到 methodHandlers
      _underscore2.default.each(methods, function (func, name) {
        if (typeof func !== 'function') {
          throw new Error("Method '" + name + "' must be a function");
        }
        if (_this.methodHandlers[name]) {
          throw new Error("A method named '" + name + "' is already defined");
        }
        _this.methodHandlers[name] = func;
      });
    },


    // server 版的 applyMethod, 返回 promise
    applyMethod: function () {
      var _ref5 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4(name, args, callback) {
        var handler, result;
        return _regenerator2.default.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                handler = this.methodHandlers[name];
                // 找不到 handler 輸出錯誤

                if (handler) {
                  _context4.next = 3;
                  break;
                }

                throw this.Error(404, 'Method \'' + name + '\' not found');

              case 3:
                _context4.prev = 3;
                _context4.t0 = _ejson.EJSON;
                _context4.next = 7;
                return handler(_ejson.EJSON.clone(args));

              case 7:
                _context4.t1 = _context4.sent;
                result = _context4.t0.clone.call(_context4.t0, _context4.t1);

                if (!callback) {
                  _context4.next = 12;
                  break;
                }

                callback(undefined, result); // eslint-disable-line
                return _context4.abrupt('return');

              case 12:
                return _context4.abrupt('return', result);

              case 15:
                _context4.prev = 15;
                _context4.t2 = _context4['catch'](3);

                if (!callback) {
                  _context4.next = 21;
                  break;
                }

                callback(_context4.t2);
                _context4.next = 22;
                break;

              case 21:
                throw _context4.t2;

              case 22:
              case 'end':
                return _context4.stop();
            }
          }
        }, _callee4, this, [[3, 15]]);
      }));

      function applyMethod(_x6, _x7, _x8) {
        return _ref5.apply(this, arguments);
      }

      return applyMethod;
    }(),


    // server 版的 callMethod, 返回 promise
    callMethod: function () {
      var _ref6 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee5(name) {
        for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
          args[_key - 1] = arguments[_key];
        }

        var callback, result;
        return _regenerator2.default.wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                // 檢查是否有 callback
                if (args.length && typeof args[args.length - 1] === 'function') {
                  callback = args.pop();
                }
                // 返回 result, 交給 applyMethod 處理
                _context5.next = 3;
                return this.applyMethod(name, args, callback);

              case 3:
                result = _context5.sent;
                return _context5.abrupt('return', result);

              case 5:
              case 'end':
                return _context5.stop();
            }
          }
        }, _callee5, this);
      }));

      function callMethod(_x9) {
        return _ref6.apply(this, arguments);
      }

      return callMethod;
    }()
  });
}

(0, _index.addModule)(methodsModule);