'use strict';

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _underscore = require('underscore');

var _underscore2 = _interopRequireDefault(_underscore);

var _extend = require('extend');

var _extend2 = _interopRequireDefault(_extend);

var _settings = require('../../lib/modules/settings.js');

var _settings2 = _interopRequireDefault(_settings);

var _index = require('./index.js');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function serverSettingsModule(server, ctx) {
  // ctx.settings 來自 create server 時的 options.settings
  ctx.settings = (0, _extend2.default)({}, ctx.settings || {}, {
    // add custom server settings here
  });

  // 先執行一般版的 settings module
  (0, _settings2.default)(server, ctx);

  // startup 時整合 database 的 settings
  // 若有錯誤，會 stop server 的 prepare, 整個 app 會 stop
  server.startup(function () {
    var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee() {
      var Settings, settings;
      return _regenerator2.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              Settings = server.createCollection('settings');
              _context.next = 3;
              return Settings.findOne();

            case 3:
              settings = _context.sent;

              if (settings) {
                (0, _extend2.default)(true, server.settings, _underscore2.default.omit(settings, 'app', '_id'));
              }

            case 5:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, this);
    }));

    function extendSettingsFromDatabase() {
      return _ref.apply(this, arguments);
    }

    return extendSettingsFromDatabase;
  }(), 15);

  // 經過整合 database 的 settings 後
  // 有可能 env 已經發生改變
  server.startup(function () {
    var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2() {
      return _regenerator2.default.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              try {
                // 處理 mailUrl
                if (server.getSetting('mailUrl') && !process.env.MAIL_URL) {
                  process.env.MAIL_URL = server.getSetting('mailUrl');
                  delete server.settings.mailUrl;
                  delete server.settings.public.mailUrl;
                }
              } catch (__) {}

            case 1:
            case 'end':
              return _context2.stop();
          }
        }
      }, _callee2, this);
    }));

    function settingsToEnv() {
      return _ref2.apply(this, arguments);
    }

    return settingsToEnv;
  }(), 16);

  // absoluteUrl 會用到 settings 的 rootUrl
  // 針對 process.env.NOW_URL 和 process.env.ROOT_URL
  // 重新宣告 server.settings.public.rootUrl
  server.startup(function () {
    var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3() {
      return _regenerator2.default.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              if (process.env.NOW_URL) {
                server.settings.public.rootUrl = process.env.NOW_URL;
              }
              if (process.env.ROOT_URL) {
                server.settings.public.rootUrl = process.env.ROOT_URL;
              }

            case 2:
            case 'end':
              return _context3.stop();
          }
        }
      }, _callee3, this);
    }));

    function envToSettings() {
      return _ref3.apply(this, arguments);
    }

    return envToSettings;
  }(), 17);
}

(0, _index.addModule)(serverSettingsModule);